import { describe, it } from 'mocha';
import { expect } from 'chai';

import { UsbTransport, filterPortsEventType, IFilterUsbPorts, ISelectedUsbPort, selectedPortEventType } from '../../ti-transport-usb/lib/UsbTransport';
import { StreamingDataModel } from '../../ti-model-streaming/lib/StreamingDataModel';
import { CodecRegistry } from '../../ti-target-configuration/lib/CodecRegistry';
import { IListener } from '../../ti-core-assets/lib/Events';
import { BootLoaderCodec } from '../lib/BootLoaderCodec';
import '../../ti-core-assets/lib/NodeJSEnv';

/* eslint-disable @typescript-eslint/no-non-null-assertion */
/* eslint-disable no-console */

describe('BootLoaderCodec', () => {

    let usbTransport: UsbTransport;
    let model: StreamingDataModel;
    let codec: BootLoaderCodec;

    before(() => {
        usbTransport = new UsbTransport({ id: 'usb', usb: true, deviceId: 'CC3220SF', defaultBaudRate: 921600 });
        model = new StreamingDataModel({ id: 'model' });
        codec = new BootLoaderCodec({ id: 'bootloader' });
        CodecRegistry.configure('usb+bootloader+model');
    });

    it('connect', async () => {
        let onFilterEventHit = false;
        let onSelectEventHit = false;

        const onFilterPorts: IListener<IFilterUsbPorts> = (details) => {
            onFilterEventHit = true;
            expect(details.ports).to.be.not.empty;
            expect(usbTransport.isConnecting()).to.be.true;

            details.ports.forEach((port) => {
                // @ts-ignore
                if (port.portInfo && port.portInfo.serialNumber && port.portInfo.serialNumber.endsWith('0')) {
                    details.recommendedPort = port;
                    details.recommendedBaudRate = 921600;
                }
            });
        };

        const onSelectedPort: IListener<ISelectedUsbPort> = (details) => {
            onSelectEventHit = true;
            expect(details.port).to.exist;
            expect(usbTransport.isConnecting()).to.be.true;
            console.log(`selectedPort = ${details.port.comName}, selectedBaudRate = ${details.baudRate}`);
        };

        usbTransport.addEventListener(filterPortsEventType, onFilterPorts);
        usbTransport.addEventListener(selectedPortEventType, onSelectedPort);

        await usbTransport.connect();
        expect(onFilterEventHit).to.be.true;
        expect(onSelectEventHit).to.be.true;
        expect(usbTransport.isConnected()).to.be.true;

        await codec.connect();
    });

    it('device status', () => {
        expect(model.getBinding('device')!.getValue()).to.equal('CC3220SF');
        expect(model.getBinding('secure')!.getValue()).to.be.true;
        expect(model.getBinding('mac_address')!.getValue()).to.equal('98:84:e3:f6:7c:44');
        expect(model.getBinding('dev_mode')!.getValue()).to.be.true;
        expect(model.getBinding('dev_not_formatted')!.getValue()).to.be.false;
        expect(model.getBinding('capacity')!.getValue()).to.equal(4);
        expect(model.getBinding('storage_size')!.getValue()).to.equal(4096);
        expect(model.getBinding('available_user_files_size')!.getValue()).to.equal(1572);
        expect(model.getBinding('sflash_codes')!.getValue()).to.deep.equal(['0xc2', '0x28', '0x16']);
        expect(model.getBinding('alerts')!.getValue()).to.deep.equal([0, 15]);
    });

    it('get file list', async () => {
        await codec.refresh();
        const userFiles = model.getBinding('userFiles')!.getValue();

        expect(userFiles.children.length).to.deep.equal(4);
        expect(userFiles.children[0].name).to.equal('@@.tmp');
        expect(userFiles.children[3].isDirectory).to.be.true;
        expect(userFiles.children[1].isDirectory).to.be.false;
        expect(userFiles.children[0].flagNoSignatureTest).to.be.true;
        expect(userFiles.children[0].flagSecure).to.be.false;
        expect(userFiles.children[1].origFileSize).to.equal(975);
        expect(userFiles.children[1].name).to.equal('dummy-root-ca-cert');
        expect(userFiles.children[2].children.length).to.equal(12);
        expect(userFiles.children[3].children.length).to.equal(2);
        expect(userFiles.children[2].children[1].flagFailSafe).to.be.true;
        expect(userFiles.children[2].children[1].flagPublicRead).to.be.false;
        expect(userFiles.children[2].children[1].flagSecure).to.be.true;
        expect(userFiles.children[2].children[1].flagPublicWrite).to.be.true;
        expect(userFiles.children[3].children[1].flagSystemFile).to.be.true;
        expect(userFiles.children[2].children[7].maxFileSize).to.equal(1052232);
        expect(userFiles.children[3].children[1].origFileSize).to.equal(0);
    });

    it('upload file', async () => {
        await codec.uploadFile('@@.tmp', 'This is a test \nOf The file upload\nFeature.');
    });

    it('download file', async () => {
        const result = await codec.downloadFile('@@.tmp');
        const textContent = (new TextDecoder()).decode(result);
        expect(textContent).to.equal('This is a test \nOf The file upload\nFeature.');
    });

    it('disconnect', async () => {
        await new Promise(resolve => setTimeout(resolve, 100));  // TODO: get rid of this hack when serial module in cloud agent is fixed.
        await codec.disconnect();
        await usbTransport.disconnect();
        expect(usbTransport.isDisconnected()).to.be.true;
    });
});